// waveshare-shield-adda-11010.js
//
// Copyright (c) 2020 Spring City Solutions LLC and other contributors
// Licensed under the Apache License, Version 2.0
// WAVESHARE (tm) is a trademark of Shenzhen Weixue Electronic Co., Ltd.
//
// https://www.waveshare.com/High-Precision-AD-DA-Board.htm
// https://www.waveshare.com/wiki/High-Precision_AD/DA_Board
// https://github.com/waveshare/High-Precision-AD-DA-Board
// https://gitlab.com/SpringCitySolutionsLLC/waveshare-shield-adda-11010
// https://npmjs.org/package/node-red-contrib-waveshare-shield-adda-11010
// https://flows.nodered.org/node/node-red-contrib-waveshare-shield-adda-11010
// https://nodered.org/
// Links for ADS1256 chip:
// https://www.ti.com/product/ADS1256
// https://www.ti.com/lit/gpn/ads1256
// Link for LM285 chip, connected to the ADS1256:
// https://www.ti.com/product/LM285-2.5-N
// https://www.ti.com/lit/gpn/lm285-2.5-n
// Link for DAC8532 chip:
// https://www.ti.com/product/DAC8532
// https://www.ti.com/lit/gpn/dac8532
//
module.exports = function(RED) {
    "use strict";
    var execSync = require('child_process').execSync;
    var fs = require('fs');

    var testCommand = __dirname + '/library-test.py';
    var adcCommand = __dirname + '/ads1256.py';
    var dacCommand = __dirname + '/dac8532.py';
    var gpioCommand = __dirname + '/gpio.py';
    var allOK = true;

    try {
        execSync(testCommand);
    } catch (err) {
        allOK = false;
        RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.libnotfound"));
    }

    // the magic to make python print stuff immediately
    process.env.PYTHONUNBUFFERED = 1;

    function ad0(n) {
        RED.nodes.createNode(this, n);
        this.out = n.out || "out";
        var node = this;
        var out = "";

        function inputlistener(msg, send, done) {
            try {
                out = execSync(adcCommand + " analog 1").toString();
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.ads1256failure"));
            }
            var scaledOut = Number(out) / 1677721.6;
            if (scaledOut < 0) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.ads1256failure"));
            } else {
                msg.payload = scaledOut;
                node.send(msg);
                if (RED.settings.verbose) { node.log("ad0 out: " + scaledOut); }
                node.status({ fill: "green", shape: "dot", text: scaledOut.toString() });
            }
            if (done) { done(); }
        }
        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);
        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }

        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            done();
        });

    }
    RED.nodes.registerType("ad0", ad0);

    function ad1(n) {
        RED.nodes.createNode(this, n);
        this.out = n.out || "out";
        var node = this;
        var out = "";

        function inputlistener(msg, send, done) {
            try {
                out = execSync(adcCommand + " analog 2").toString();
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.ads1256failure"));
            }
            var scaledOut = Number(out) / 1677721.6;
            if (scaledOut < 0) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.ads1256failure"));
            } else {
                msg.payload = scaledOut;
                node.send(msg);
                if (RED.settings.verbose) { node.log("ad0 out: " + scaledOut); }
                node.status({ fill: "green", shape: "dot", text: scaledOut.toString() });
            }
            if (done) { done(); }
        }
        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);
        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }

        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            done();
        });

    }
    RED.nodes.registerType("ad1", ad1);

    function ad2(n) {
        RED.nodes.createNode(this, n);
        this.out = n.out || "out";
        var node = this;
        var out = "";

        function inputlistener(msg, send, done) {
            try {
                out = execSync(adcCommand + " analog 3").toString();
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.ads1256failure"));
            }
            var scaledOut = Number(out) / 1677721.6;
            if (scaledOut < 0) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.ads1256failure"));
            } else {
                msg.payload = scaledOut;
                node.send(msg);
                if (RED.settings.verbose) { node.log("ad0 out: " + scaledOut); }
                node.status({ fill: "green", shape: "dot", text: scaledOut.toString() });
            }
            if (done) { done(); }
        }
        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);
        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }

        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            done();
        });

    }
    RED.nodes.registerType("ad2", ad2);

    function ad3(n) {
        RED.nodes.createNode(this, n);
        this.out = n.out || "out";
        var node = this;
        var out = "";

        function inputlistener(msg, send, done) {
            try {
                out = execSync(adcCommand + " analog 4").toString();
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.ads1256failure"));
            }
            var scaledOut = Number(out) / 1677721.6;
            if (scaledOut < 0) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.ads1256failure"));
            } else {
                msg.payload = scaledOut;
                node.send(msg);
                if (RED.settings.verbose) { node.log("ad0 out: " + scaledOut); }
                node.status({ fill: "green", shape: "dot", text: scaledOut.toString() });
            }
            if (done) { done(); }
        }
        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);
        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }

        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            done();
        });

    }
    RED.nodes.registerType("ad3", ad3);

    function ad4(n) {
        RED.nodes.createNode(this, n);
        this.out = n.out || "out";
        var node = this;
        var out = "";

        function inputlistener(msg, send, done) {
            try {
                out = execSync(adcCommand + " analog 5").toString();
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.ads1256failure"));
            }
            var scaledOut = Number(out) / 1677721.6;
            if (scaledOut < 0) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.ads1256failure"));
            } else {
                msg.payload = scaledOut;
                node.send(msg);
                if (RED.settings.verbose) { node.log("ad0 out: " + scaledOut); }
                node.status({ fill: "green", shape: "dot", text: scaledOut.toString() });
            }
            if (done) { done(); }
        }
        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);
        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }

        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            done();
        });

    }
    RED.nodes.registerType("ad4", ad4);

    function ad5(n) {
        RED.nodes.createNode(this, n);
        this.out = n.out || "out";
        var node = this;
        var out = "";

        function inputlistener(msg, send, done) {
            try {
                out = execSync(adcCommand + " analog 6").toString();
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.ads1256failure"));
            }
            var scaledOut = Number(out) / 1677721.6;
            if (scaledOut < 0) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.ads1256failure"));
            } else {
                msg.payload = scaledOut;
                node.send(msg);
                if (RED.settings.verbose) { node.log("ad0 out: " + scaledOut); }
                node.status({ fill: "green", shape: "dot", text: scaledOut.toString() });
            }
            if (done) { done(); }
        }
        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);
        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }

        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            done();
        });

    }
    RED.nodes.registerType("ad5", ad5);

    function ad6(n) {
        RED.nodes.createNode(this, n);
        this.out = n.out || "out";
        var node = this;
        var out = "";

        function inputlistener(msg, send, done) {
            try {
                out = execSync(adcCommand + " analog 7").toString();
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.ads1256failure"));
            }
            var scaledOut = Number(out) / 1677721.6;
            if (scaledOut < 0) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.ads1256failure"));
            } else {
                msg.payload = scaledOut;
                node.send(msg);
                if (RED.settings.verbose) { node.log("ad0 out: " + scaledOut); }
                node.status({ fill: "green", shape: "dot", text: scaledOut.toString() });
            }
            if (done) { done(); }
        }
        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);
        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }

        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            done();
        });

    }
    RED.nodes.registerType("ad6", ad6);

    function ad7(n) {
        RED.nodes.createNode(this, n);
        this.out = n.out || "out";
        var node = this;
        var out = "";

        function inputlistener(msg, send, done) {
            try {
                out = execSync(adcCommand + " analog 8").toString();
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.ads1256failure"));
            }
            var scaledOut = Number(out) / 1677721.6;
            if (scaledOut < 0) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.ads1256failure"));
            } else {
                msg.payload = scaledOut;
                node.send(msg);
                if (RED.settings.verbose) { node.log("ad0 out: " + scaledOut); }
                node.status({ fill: "green", shape: "dot", text: scaledOut.toString() });
            }
            if (done) { done(); }
        }
        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);
        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }

        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            done();
        });

    }
    RED.nodes.registerType("ad7", ad7);

    function dac0(n) {
        RED.nodes.createNode(this, n);
        this.scale = n.scale;
        this.out = n.out || "out";
        var node = this;

        function inputlistener(msg, send, done) {
            var out = Number(msg.payload);
            if (node.scale == 'vref-33') { out = 19859.4 * out };
            if (node.scale == 'vref-50') { out = 13107.2 * out };
            if (out < 0) { out = 0 };
            if (out > 65535) { out = 65535 };
            out = parseInt(out).toString();
            if (RED.settings.verbose) { node.log("dac0 out: " + out); }
            node.status({ fill: "green", shape: "dot", text: msg.payload.toString() });
            try {
                execSync(dacCommand + " A " + out);
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.dac8532failure"));
            }
            if (done) { done(); }

        }

        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);

        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }

        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            execSync(dacCommand + " A " + "0");
            done();
        });

    }
    RED.nodes.registerType("dac0", dac0);

    function dac1(n) {
        RED.nodes.createNode(this, n);
        this.scale = n.scale;
        this.out = n.out || "out";
        var node = this;

        function inputlistener(msg, send, done) {
            var out = Number(msg.payload);
            if (node.scale == 'vref-33') { out = 19859.4 * out };
            if (node.scale == 'vref-50') { out = 13107.2 * out };
            if (out < 0) { out = 0 };
            if (out > 65535) { out = 65535 };
            out = parseInt(out).toString();
            if (RED.settings.verbose) { node.log("dac1 out: " + out); }
            node.status({ fill: "green", shape: "dot", text: msg.payload.toString() });
            try {
                execSync(dacCommand + " B " + out);
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.dac8532failure"));
            }
            if (done) { done(); }

        }

        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);

        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }

        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            execSync(dacCommand + " B " + "0");
            done();
        });

    }
    RED.nodes.registerType("dac1", dac1);

    function gpiod0in(n) {
        RED.nodes.createNode(this, n);
        this.out = n.out || "out";
        var node = this;
        var out = "";

        function inputlistener(msg, send, done) {
            try {
                out = execSync(adcCommand + " digitalIn D0").toString();
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.ads1256failure"));
            }
            msg.payload = Number(out);
            node.send(msg);
            if (RED.settings.verbose) { node.log("gpiod0 in: " + scaledOut); }
            node.status({ fill: "green", shape: "dot", text: scaledOut.toString() });
            if (done) { done(); }
        }
        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);
        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }
        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            done();
        });
    }
    RED.nodes.registerType("gpiod0in", gpiod0in);

    function gpiod0out(n) {
        RED.nodes.createNode(this, n);
        this.scale = n.scale;
        this.out = n.out || "out";
        var node = this;

        function inputlistener(msg, send, done) {
            var out = Number(msg.payload);
            if (out < 0) { out = 0 };
            if (out > 1) { out = 1 };
            out = parseInt(out).toString();
            if (RED.settings.verbose) { node.log("gpiod0 out: " + out); }
            node.status({ fill: "green", shape: "dot", text: msg.payload.toString() });
            try {
                execSync(adcCommand + " digitalOut D0 " + out);
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.ads1256failure"));
            }
            if (done) { done(); }

        }

        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);

        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }

        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            done();
        });

    }
    RED.nodes.registerType("gpiod0out", gpiod0out);

    function gpiod1in(n) {
        RED.nodes.createNode(this, n);
        this.out = n.out || "out";
        var node = this;
        var out = "";

        function inputlistener(msg, send, done) {
            try {
                out = execSync(adcCommand + " digitalIn D1").toString();
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.ads1256failure"));
            }
            msg.payload = Number(out);
            node.send(msg);
            if (RED.settings.verbose) { node.log("gpiod1 in: " + scaledOut); }
            node.status({ fill: "green", shape: "dot", text: scaledOut.toString() });
            if (done) { done(); }
        }
        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);
        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }
        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            done();
        });
    }
    RED.nodes.registerType("gpiod1in", gpiod1in);

    function gpiod1out(n) {
        RED.nodes.createNode(this, n);
        this.scale = n.scale;
        this.out = n.out || "out";
        var node = this;

        function inputlistener(msg, send, done) {
            var out = Number(msg.payload);
            if (out < 0) { out = 0 };
            if (out > 1) { out = 1 };
            out = parseInt(out).toString();
            if (RED.settings.verbose) { node.log("gpiod1 out: " + out); }
            node.status({ fill: "green", shape: "dot", text: msg.payload.toString() });
            try {
                execSync(adcCommand + " digitalOut D1 " + out);
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.ads1256failure"));
            }
            if (done) { done(); }

        }

        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);

        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }

        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            done();
        });

    }
    RED.nodes.registerType("gpiod1out", gpiod1out);

    function gpiod2in(n) {
        RED.nodes.createNode(this, n);
        this.out = n.out || "out";
        var node = this;
        var out = "";

        function inputlistener(msg, send, done) {
            try {
                out = execSync(adcCommand + " digitalIn D2").toString();
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.ads1256failure"));
            }
            msg.payload = Number(out);
            node.send(msg);
            if (RED.settings.verbose) { node.log("gpiod2 in: " + scaledOut); }
            node.status({ fill: "green", shape: "dot", text: scaledOut.toString() });
            if (done) { done(); }
        }
        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);
        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }
        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            done();
        });
    }
    RED.nodes.registerType("gpiod2in", gpiod2in);

    function gpiod2out(n) {
        RED.nodes.createNode(this, n);
        this.scale = n.scale;
        this.out = n.out || "out";
        var node = this;

        function inputlistener(msg, send, done) {
            var out = Number(msg.payload);
            if (out < 0) { out = 0 };
            if (out > 1) { out = 1 };
            out = parseInt(out).toString();
            if (RED.settings.verbose) { node.log("gpiod2 out: " + out); }
            node.status({ fill: "green", shape: "dot", text: msg.payload.toString() });
            try {
                execSync(adcCommand + " digitalOut D2 " + out);
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.ads1256failure"));
            }
            if (done) { done(); }

        }

        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);

        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }

        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            done();
        });

    }
    RED.nodes.registerType("gpiod2out", gpiod2out);

    function gpiod3in(n) {
        RED.nodes.createNode(this, n);
        this.out = n.out || "out";
        var node = this;
        var out = "";

        function inputlistener(msg, send, done) {
            try {
                out = execSync(adcCommand + " digitalIn D3").toString();
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.ads1256failure"));
            }
            msg.payload = Number(out);
            node.send(msg);
            if (RED.settings.verbose) { node.log("gpiod1 in: " + scaledOut); }
            node.status({ fill: "green", shape: "dot", text: scaledOut.toString() });
            if (done) { done(); }
        }
        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);
        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }
        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            done();
        });
    }
    RED.nodes.registerType("gpiod3in", gpiod3in);

    function gpiod3out(n) {
        RED.nodes.createNode(this, n);
        this.scale = n.scale;
        this.out = n.out || "out";
        var node = this;

        function inputlistener(msg, send, done) {
            var out = Number(msg.payload);
            if (out < 0) { out = 0 };
            if (out > 1) { out = 1 };
            out = parseInt(out).toString();
            if (RED.settings.verbose) { node.log("gpiod3 out: " + out); }
            node.status({ fill: "green", shape: "dot", text: msg.payload.toString() });
            try {
                execSync(adcCommand + " digitalOut D3 " + out);
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.ads1256failure"));
            }
            if (done) { done(); }

        }

        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);

        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }

        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            done();
        });

    }
    RED.nodes.registerType("gpiod3out", gpiod3out);

    function gpiop22in(n) {
        RED.nodes.createNode(this, n);
        this.out = n.out || "out";
        var node = this;
        var out = "";

        function inputlistener(msg, send, done) {
            try {
                out = execSync(gpioCommand + " in P22").toString();
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.gpiofailure"));
            }
            msg.payload = Number(out);
            node.send(msg);
            if (RED.settings.verbose) { node.log("gpiop22 in: " + scaledOut); }
            node.status({ fill: "green", shape: "dot", text: scaledOut.toString() });
            if (done) { done(); }
        }
        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);
        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }
        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            done();
        });
    }
    RED.nodes.registerType("gpiop22in", gpiop22in);

    function gpiop22out(n) {
        RED.nodes.createNode(this, n);
        this.scale = n.scale;
        this.out = n.out || "out";
        var node = this;

        function inputlistener(msg, send, done) {
            var out = Number(msg.payload);
            if (out < 0) { out = 0 };
            if (out > 1) { out = 1 };
            out = parseInt(out).toString();
            if (RED.settings.verbose) { node.log("gpiop22 out: " + out); }
            node.status({ fill: "green", shape: "dot", text: msg.payload.toString() });
            try {
                execSync(gpioCommand + " out P22 " + out);
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.gpiofailure"));
            }
            if (done) { done(); }

        }

        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);

        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }

        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            done();
        });

    }
    RED.nodes.registerType("gpiop22out", gpiop22out);

    function gpiop23in(n) {
        RED.nodes.createNode(this, n);
        this.out = n.out || "out";
        var node = this;
        var out = "";

        function inputlistener(msg, send, done) {
            try {
                out = execSync(gpioCommand + " in P23").toString();
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.gpiofailure"));
            }
            msg.payload = Number(out);
            node.send(msg);
            if (RED.settings.verbose) { node.log("gpiop23 in: " + scaledOut); }
            node.status({ fill: "green", shape: "dot", text: scaledOut.toString() });
            if (done) { done(); }
        }
        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);
        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }
        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            done();
        });
    }
    RED.nodes.registerType("gpiop23in", gpiop23in);

    function gpiop23out(n) {
        RED.nodes.createNode(this, n);
        this.scale = n.scale;
        this.out = n.out || "out";
        var node = this;

        function inputlistener(msg, send, done) {
            var out = Number(msg.payload);
            if (out < 0) { out = 0 };
            if (out > 1) { out = 1 };
            out = parseInt(out).toString();
            if (RED.settings.verbose) { node.log("gpiop23 out: " + out); }
            node.status({ fill: "green", shape: "dot", text: msg.payload.toString() });
            try {
                execSync(gpioCommand + " out P23 " + out);
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.gpiofailure"));
            }
            if (done) { done(); }

        }

        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);

        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }

        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            done();
        });

    }
    RED.nodes.registerType("gpiop23out", gpiop23out);

    function gpiop24in(n) {
        RED.nodes.createNode(this, n);
        this.out = n.out || "out";
        var node = this;
        var out = "";

        function inputlistener(msg, send, done) {
            try {
                out = execSync(gpioCommand + " in P24").toString();
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.gpiofailure"));
            }
            msg.payload = Number(out);
            node.send(msg);
            if (RED.settings.verbose) { node.log("gpiop24 in: " + scaledOut); }
            node.status({ fill: "green", shape: "dot", text: scaledOut.toString() });
            if (done) { done(); }
        }
        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);
        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }
        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            done();
        });
    }
    RED.nodes.registerType("gpiop24in", gpiop24in);

    function gpiop24out(n) {
        RED.nodes.createNode(this, n);
        this.scale = n.scale;
        this.out = n.out || "out";
        var node = this;

        function inputlistener(msg, send, done) {
            var out = Number(msg.payload);
            if (out < 0) { out = 0 };
            if (out > 1) { out = 1 };
            out = parseInt(out).toString();
            if (RED.settings.verbose) { node.log("gpiop24 out: " + out); }
            node.status({ fill: "green", shape: "dot", text: msg.payload.toString() });
            try {
                execSync(gpioCommand + " out P24 " + out);
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.gpiofailure"));
            }
            if (done) { done(); }

        }

        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);

        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }

        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            done();
        });

    }
    RED.nodes.registerType("gpiop24out", gpiop24out);


    function gpiop25in(n) {
        RED.nodes.createNode(this, n);
        this.out = n.out || "out";
        var node = this;
        var out = "";

        function inputlistener(msg, send, done) {
            try {
                out = execSync(gpioCommand + " in P25").toString();
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.gpiofailure"));
            }
            msg.payload = Number(out);
            node.send(msg);
            if (RED.settings.verbose) { node.log("gpiop25 in: " + scaledOut); }
            node.status({ fill: "green", shape: "dot", text: scaledOut.toString() });
            if (done) { done(); }
        }
        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);
        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }
        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            done();
        });
    }
    RED.nodes.registerType("gpiop25in", gpiop25in);

    function gpiop25out(n) {
        RED.nodes.createNode(this, n);
        this.scale = n.scale;
        this.out = n.out || "out";
        var node = this;

        function inputlistener(msg, send, done) {
            var out = Number(msg.payload);
            if (out < 0) { out = 0 };
            if (out > 1) { out = 1 };
            out = parseInt(out).toString();
            if (RED.settings.verbose) { node.log("gpiop25 out: " + out); }
            node.status({ fill: "green", shape: "dot", text: msg.payload.toString() });
            try {
                execSync(gpioCommand + " out P25 " + out);
            } catch (err) {
                allOK = false;
                node.status({ fill: "red", shape: "ring", text: "waveshare-shield-adda-11010.status.not-running" });
                RED.log.warn("waveshare-shield-adda-11010 : " + RED._("waveshare-shield-adda-11010.errors.gpiofailure"));
            }
            if (done) { done(); }

        }

        if (allOK === true) {
            node.running = true;
            node.on("input", inputlistener);

        } else {
            node.status({ fill: "grey", shape: "dot", text: "waveshare-shield-adda-11010.status.not-available" });
            node.on("input", function(msg) {
                node.status({ fill: "grey", shape: "dot", text: RED._("waveshare-shield-adda-11010.status.na", { value: msg.payload.toString() }) });
            });
        }

        node.on("close", function(done) {
            node.status({ fill: "grey", shape: "ring", text: "waveshare-shield-adda-11010.status.closed" });
            done();
        });

    }
    RED.nodes.registerType("gpiop25out", gpiop25out);

}